# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from contextlib import closing
from os import stat
try:
    from importlib.resources import files
except ImportError:
    from importlib_resources import files
from pathlib import Path
import shutil
import tempfile
import time
import unittest
from zipfile import ZipFile, ZIP_DEFLATED

from pyface.image_resource import ImageResource
from pyface.ui_traits import Border, Margin
from ..image import (
    FastZipFile, ImageLibrary, ImageVolume, ImageVolumeInfo, ZipFileReference,
    join_image_name, split_image_name, time_stamp_for,
)


LIBRARY_DIR = files('pyface.image') / "library"
ICONS_FILE = LIBRARY_DIR / "icons.zip"
TEST_IMAGES_DIR = files('pyface.tests') / "images"


class TestJoinImageName(unittest.TestCase):

    def test_simple(self):
        image_name = join_image_name("icons", "red_ball.jpg")

        self.assertEqual(image_name, "@icons:red_ball.jpg")

    def test_extension(self):
        image_name = join_image_name("icons", "red_ball.png")

        self.assertEqual(image_name, "@icons:red_ball")

    def test_double_extension(self):
        image_name = join_image_name("icons", "red_ball.foo.png")

        self.assertEqual(image_name, "@icons:red_ball.foo.png")


class TestSplitImageName(unittest.TestCase):

    def test_simple(self):
        volume_name, file_name = split_image_name("@icons:red_ball.jpg")

        self.assertEqual(volume_name, "icons")
        self.assertEqual(file_name, "red_ball.jpg")

    def test_extension(self):
        volume_name, file_name = split_image_name("@icons:red_ball")

        self.assertEqual(volume_name, "icons")
        self.assertEqual(file_name, "red_ball.png")

    def test_no_at_symbol(self):
        volume_name, file_name = split_image_name("icons:red_ball.jpg")

        # XXX this should probably raise rather than doing this
        self.assertEqual(volume_name, "cons")
        self.assertEqual(file_name, "red_ball.jpg")

    def test_no_colon(self):
        volume_name, file_name = split_image_name("@red_ball.jpg")

        # XXX this should probably raise rather than doing this
        self.assertEqual(volume_name, "red_ball.jp")
        self.assertEqual(file_name, "@red_ball.jpg")

    def test_no_colon_or_at_symbol(self):
        volume_name, file_name = split_image_name("red_ball.jpg")

        # XXX this should probably raise rather than doing this
        self.assertEqual(volume_name, "ed_ball.jp")
        self.assertEqual(file_name, "red_ball.jpg")


class TestFastZipFile(unittest.TestCase):

    def test_read_icons_red_ball(self):
        zf = FastZipFile(path=ICONS_FILE)

        file_bytes = zf.read("red_ball.png")

        self.assertTrue(file_bytes.startswith(b"\x89PNG"))

    def test_namelist_icons(self):
        zf = FastZipFile(path=ICONS_FILE)

        names = zf.namelist()

        self.assertTrue("red_ball.png" in names)
        self.assertEqual(names, zf.zf.namelist())

    def test_close_icons(self):
        zf = FastZipFile(path=ICONS_FILE)

        actual_zf = zf.zf

        self.assertIs(zf._zf, actual_zf)

        zf.close()

        self.assertIsNone(zf._zf)

    def test_eventual_zipfile_close(self):
        zf = FastZipFile(path=ICONS_FILE)

        self.assertFalse(zf._running)
        start_time = time.time()

        actual_zf = zf.zf

        end_time = time.time()

        self.assertIsNotNone(actual_zf)
        self.assertGreaterEqual(zf.time_stamp, start_time)
        self.assertLessEqual(zf.time_stamp, end_time)
        self.assertTrue(zf._running)
        self.assertIs(zf._zf, actual_zf)

        # wait for thread to clean-up zipfile
        # XXX this is not nice
        while time.time() <= end_time + 3.0:
            time.sleep(1.0)

        self.assertFalse(zf._running)
        self.assertIsNone(zf._zf)


class TestImageVolume(unittest.TestCase):

    def test_init_empty(self):
        volume = ImageVolume()

        self.assertEqual(volume.name, "")
        self.assertEqual(volume.images, [])
        self.assertEqual(volume.catalog, {})
        self.assertEqual(volume.category, "General")
        self.assertEqual(volume.keywords, [])
        self.assertEqual(volume.aliases, [])
        self.assertEqual(volume.path, "")
        self.assertTrue(volume.is_zip_file)
        self.assertIsNone(volume.zip_file)
        self.assertEqual(volume.time_stamp, "")
        self.assertEqual(len(volume.info), 1)
        self.assertIsInstance(volume.info[0], ImageVolumeInfo)

    def test_empty_zipfile(self):
        with tempfile.TemporaryDirectory() as dir_path:
            path = Path(dir_path) / "test.zip"
            ZipFile(path, "w", ZIP_DEFLATED).close()
            with closing(FastZipFile(path=path)) as zf:
                time_stamp = time_stamp_for(stat(path).st_mtime)

                volume = ImageVolume(name="test", path=path, zip_file=zf)

                self.assertEqual(volume.name, "test")
                self.assertEqual(volume.images, [])
                self.assertEqual(volume.catalog, {})
                self.assertEqual(volume.category, "General")
                self.assertEqual(volume.keywords, [])
                self.assertEqual(volume.aliases, [])
                self.assertEqual(Path(volume.path), path)
                self.assertTrue(volume.is_zip_file)
                self.assertIs(volume.zip_file, zf)
                self.assertEqual(volume.time_stamp, time_stamp)
                self.assertEqual(len(volume.info), 1)
                self.assertIsInstance(volume.info[0], ImageVolumeInfo)

    def test_empty_directory(self):
        with tempfile.TemporaryDirectory() as dir_path:
            path = Path(dir_path) / "test"
            path.mkdir()
            time_stamp = time_stamp_for(stat(path).st_mtime)

            volume = ImageVolume(name="test", path=path, is_zip_file=False)

            self.assertEqual(volume.name, "test")
            self.assertEqual(volume.images, [])
            self.assertEqual(volume.catalog, {})
            self.assertEqual(volume.category, "General")
            self.assertEqual(volume.keywords, [])
            self.assertEqual(volume.aliases, [])
            self.assertEqual(Path(volume.path), path)
            self.assertFalse(volume.is_zip_file)
            self.assertIsNone(volume.zip_file)
            self.assertEqual(volume.time_stamp, time_stamp)
            self.assertEqual(len(volume.info), 1)
            self.assertIsInstance(volume.info[0], ImageVolumeInfo)

    def test_empty_directory_save(self):
        with tempfile.TemporaryDirectory() as dir_path:
            path = Path(dir_path) / "test"
            path.mkdir()
            time_stamp = time_stamp_for(stat(path).st_mtime)

            volume = ImageVolume(name="test", path=path, is_zip_file=False)
            result = volume.save()

            self.assertTrue(result)

            filenames = {file.name for file in path.iterdir()}
            self.assertEqual(filenames, {"image_volume.py", "image_info.py", "license.txt"})

            # test that new file is readable
            time_stamp = time_stamp_for(stat(path).st_mtime)

            volume_2 = ImageVolume(name="test", path=path, is_zip_file=False)

            self.assertEqual(volume_2.name, "test")
            self.assertEqual(volume_2.images, [])
            self.assertEqual(volume_2.catalog, {})
            self.assertEqual(volume_2.category, "General")
            self.assertEqual(volume_2.keywords, [])
            self.assertEqual(volume_2.aliases, [])
            self.assertEqual(Path(volume_2.path), path)
            self.assertFalse(volume_2.is_zip_file)
            self.assertIsNone(volume_2.zip_file)
            self.assertEqual(volume_2.time_stamp, time_stamp)
            self.assertEqual(len(volume_2.info), 1)
            self.assertIsInstance(volume_2.info[0], ImageVolumeInfo)

    def test_empty_zipfile_save(self):
        with tempfile.TemporaryDirectory() as dir_path:
            path = Path(dir_path) / "test.zip"
            ZipFile(path, "w", ZIP_DEFLATED).close()
            with closing(FastZipFile(path=path)) as zf:
                time_stamp = time_stamp_for(stat(path).st_mtime)

                volume = ImageVolume(name="test", path=path, zip_file=zf)

                result = volume.save()

            self.assertTrue(result)

            with closing(FastZipFile(path=path)) as zf:
                self.assertEqual(set(zf.namelist()), {"image_volume.py", "image_info.py", "license.txt"})

            # test that new file is readable
            with closing(FastZipFile(path=path)) as zf:
                time_stamp = time_stamp_for(stat(path).st_mtime)

                volume_2 = ImageVolume(name="test", path=path, zip_file=zf)

                self.assertEqual(volume_2.name, "test")
                self.assertEqual(volume_2.images, [])
                self.assertEqual(volume_2.catalog, {})
                self.assertEqual(volume_2.category, "General")
                self.assertEqual(volume_2.keywords, [])
                self.assertEqual(volume_2.aliases, [])
                self.assertEqual(Path(volume_2.path), path)
                self.assertTrue(volume_2.is_zip_file)
                self.assertIs(volume_2.zip_file, zf)
                self.assertEqual(volume_2.time_stamp, time_stamp)
                self.assertEqual(len(volume_2.info), 1)
                self.assertIsInstance(volume_2.info[0], ImageVolumeInfo)

    def test_save_directory(self):
        with tempfile.TemporaryDirectory() as dir_path:
            path = Path(dir_path) / "test"
            path.mkdir()
            time_stamp = time_stamp_for(stat(path).st_mtime)
            image_file = path / "core.png"
            shutil.copyfile(TEST_IMAGES_DIR / "core.png", image_file)

            # this should create an default ImageInfo object for the image
            # file as a side-effect of loading.
            volume = ImageVolume(name="test", path=path, is_zip_file=False)

            self.assertEqual(len(volume.images), 1)
            self.assertGreaterEqual(volume.time_stamp, time_stamp)

            image = volume.images[0]
            self.assertEqual(image.image_name, "@test:core")
            self.assertEqual(image.name, "core")
            self.assertEqual(image.volume, volume)
            self.assertEqual(image.description, "")
            self.assertEqual(image.category, "General")
            self.assertEqual(image.keywords, [])
            self.assertEqual(image.width, 64)
            self.assertEqual(image.height, 64)
            self.assertIsInstance(image.border, Border)
            self.assertIsInstance(image.content, Margin)
            self.assertIsInstance(image.label, Margin)
            self.assertEqual(image.alignment, "default")
            self.assertEqual(image.copyright, 'No copyright information specified.')

            result = volume.save()

            self.assertTrue(result)

            filenames = {file.name for file in path.iterdir()}
            self.assertEqual(filenames, {"core.png", "image_volume.py", "image_info.py", "license.txt"})

            # test that new file is readable
            time_stamp = time_stamp_for(stat(path).st_mtime)

            volume_2 = ImageVolume(name="test", path=path, is_zip_file=False)

            self.assertEqual(volume_2.name, "test")
            self.assertEqual(len(volume_2.images), 1)
            self.assertEqual(len(volume_2.catalog), 1)
            self.assertIn("@test:core", volume_2.catalog)
            self.assertEqual(volume_2.category, "General")
            self.assertEqual(volume_2.keywords, [])
            self.assertEqual(volume_2.aliases, [])
            self.assertEqual(Path(volume_2.path), path)
            self.assertFalse(volume_2.is_zip_file)
            self.assertIsNone(volume_2.zip_file)
            self.assertEqual(volume_2.time_stamp, time_stamp)
            self.assertEqual(len(volume_2.info), 1)
            self.assertIsInstance(volume_2.info[0], ImageVolumeInfo)

            image = volume_2.images[0]
            self.assertEqual(image.image_name, "@test:core")
            self.assertEqual(image.name, "core")
            self.assertEqual(image.volume, volume_2)
            self.assertEqual(image.description, "")
            self.assertEqual(image.category, "General")
            self.assertEqual(image.keywords, [])
            self.assertEqual(image.width, 64)
            self.assertEqual(image.height, 64)
            self.assertIsInstance(image.border, Border)
            self.assertIsInstance(image.content, Margin)
            self.assertIsInstance(image.label, Margin)
            self.assertEqual(image.alignment, "default")
            self.assertEqual(image.copyright, 'No copyright information specified.')

            # do one more save to smoke-test other code paths
            volume_2.save()

    def test_save_zipfile(self):
        with tempfile.TemporaryDirectory() as dir_path:
            path = Path(dir_path) / "test.zip"
            with ZipFile(path, "w", ZIP_DEFLATED) as zf:
                zf.write(TEST_IMAGES_DIR / "core.png", "core.png")

            with closing(FastZipFile(path=path)) as zf:
                time_stamp = time_stamp_for(stat(path).st_mtime)

                # this should create an default ImageInfo object for the image
                # file as a side-effect of loading.
                volume = ImageVolume(name="test", path=path, zip_file=zf)

                self.assertEqual(len(volume.images), 1)
                self.assertGreaterEqual(volume.time_stamp, time_stamp)

                image = volume.images[0]
                self.assertEqual(image.image_name, "@test:core")
                self.assertEqual(image.name, "core")
                self.assertEqual(image.volume, volume)
                self.assertEqual(image.description, "")
                self.assertEqual(image.category, "General")
                self.assertEqual(image.keywords, [])
                self.assertEqual(image.width, 64)
                self.assertEqual(image.height, 64)
                self.assertIsInstance(image.border, Border)
                self.assertIsInstance(image.content, Margin)
                self.assertIsInstance(image.label, Margin)
                self.assertEqual(image.alignment, "default")
                self.assertEqual(image.copyright, 'No copyright information specified.')

                result = volume.save()

            self.assertTrue(result)

            with closing(FastZipFile(path=path)) as zf:
                self.assertEqual(set(zf.namelist()), {"core.png", "image_volume.py", "image_info.py", "license.txt"})

            # test that new file is readable
            with closing(FastZipFile(path=path)) as zf:
                time_stamp = time_stamp_for(stat(path).st_mtime)

                volume_2 = ImageVolume(name="test", path=path, zip_file=zf)

                self.assertEqual(volume_2.name, "test")
                self.assertEqual(len(volume_2.images), 1)
                self.assertEqual(len(volume_2.catalog), 1)
                self.assertIn("@test:core", volume_2.catalog)
                self.assertEqual(volume_2.category, "General")
                self.assertEqual(volume_2.keywords, [])
                self.assertEqual(volume_2.aliases, [])
                self.assertEqual(Path(volume_2.path), path)
                self.assertTrue(volume_2.is_zip_file)
                self.assertIs(volume_2.zip_file, zf)
                self.assertEqual(volume_2.time_stamp, time_stamp)
                self.assertEqual(len(volume_2.info), 1)
                self.assertIsInstance(volume_2.info[0], ImageVolumeInfo)

                image = volume_2.images[0]
                self.assertEqual(image.image_name, "@test:core")
                self.assertEqual(image.name, "core")
                self.assertEqual(image.volume, volume_2)
                self.assertEqual(image.description, "")
                self.assertEqual(image.category, "General")
                self.assertEqual(image.keywords, [])
                self.assertEqual(image.height, 64)
                self.assertEqual(image.width, 64)
                self.assertIsInstance(image.border, Border)
                self.assertIsInstance(image.content, Margin)
                self.assertIsInstance(image.label, Margin)
                self.assertEqual(image.alignment, "default")
                self.assertEqual(image.copyright, 'No copyright information specified.')

                # do one more save to smoke-test other code paths
                volume_2.save()

    def test_icons_zipfile_volume(self):
        time_stamp = time_stamp_for(stat(ICONS_FILE).st_mtime)
        with closing(FastZipFile(path=ICONS_FILE)) as zf:
            volume = ImageVolume(name="icons", path=ICONS_FILE, zip_file=zf)

            self.assertEqual(volume.name, "icons")
            self.assertTrue(
                any(
                    image.image_name == "@icons:red_ball"
                    for image in volume.images
                )
            )
            self.assertTrue("@icons:red_ball" in volume.catalog)
            self.assertEqual(volume.category, "General")
            self.assertEqual(volume.keywords, [])
            self.assertEqual(volume.aliases, [])
            self.assertEqual(Path(volume.path), ICONS_FILE)
            self.assertTrue(volume.is_zip_file)
            self.assertIs(volume.zip_file, zf)
            self.assertEqual(volume.time_stamp, time_stamp)
            self.assertEqual(len(volume.info), 1)
            self.assertIsInstance(volume.info[0], ImageVolumeInfo)

    def test_icons_image_resource(self):
        with closing(FastZipFile(path=ICONS_FILE)) as zf:
            volume = ImageVolume(name="icons", path=ICONS_FILE, zip_file=zf)

            image = volume.image_resource("@icons:red_ball")

        self.assertIsInstance(image, ImageResource)
        self.assertIsInstance(image._ref, ZipFileReference)
        self.assertTrue(ICONS_FILE.samefile(image._ref.path))

    def test_icons_image_resource_missing(self):
        with closing(FastZipFile(path=ICONS_FILE)) as zf:
            volume = ImageVolume(name="icons", path=ICONS_FILE, zip_file=zf)

            image = volume.image_resource("@icons:does_not_exist")

        self.assertIsInstance(image, ImageResource)
        self.assertIsInstance(image._ref, ZipFileReference)
        self.assertTrue(ICONS_FILE.samefile(image._ref.path))

    def test_icons_image_data(self):
        with closing(FastZipFile(path=ICONS_FILE)) as zf:
            volume = ImageVolume(name="icons", path=ICONS_FILE, zip_file=zf)

            data = volume.image_data("@icons:red_ball")

        self.assertTrue(data.startswith(b"\x89PNG"))

    def test_icons_image_data_missing(self):
        with closing(FastZipFile(path=ICONS_FILE)) as zf:
            volume = ImageVolume(name="icons", path=ICONS_FILE, zip_file=zf)

            with self.assertRaises(KeyError):
                volume.image_data("@icons:does_not_exist")

    def test_icons_volume_info(self):
        with closing(FastZipFile(path=ICONS_FILE)) as zf:
            volume = ImageVolume(name="icons", path=ICONS_FILE, zip_file=zf)

            volume_info = volume.volume_info("@icons:red_ball")

        self.assertIs(volume_info, volume.info[0])

    def test_volume_info(self):
        volume = ImageVolume(
            name="test",
            info=[
                ImageVolumeInfo(image_names=["@test:one", "@test:two"]),
                ImageVolumeInfo(image_names=["@test:three", "@test:two"]),
            ]
        )

        volume_info = volume.volume_info("@test:two")
        self.assertIs(volume_info, volume.info[0])

        volume_info = volume.volume_info("@test:three")
        self.assertIs(volume_info, volume.info[1])

        with self.assertRaises(ValueError):
            volume.volume_info("@test:four")


class TestImageLibrary(unittest.TestCase):

    # XXX These are more in the flavor of integration tests

    def test_default_volumes(self):
        volume_names = {volume.name for volume in ImageLibrary.volumes}
        self.assertEqual(volume_names, {'icons', 'std'})

    def test_default_catalog(self):
        self.assertEqual(set(ImageLibrary.catalog.keys()), {'icons', 'std'})

    def test_default_images(self):
        image_names = {image.image_name for image in ImageLibrary.images}
        self.assertTrue("@icons:red_ball" in image_names)

    def test_image_info(self):
        red_ball_image_info = ImageLibrary.image_info("@icons:red_ball")

        self.assertEqual(red_ball_image_info.name, "red_ball")
        self.assertEqual(red_ball_image_info.image_name, "@icons:red_ball")
        self.assertEqual(red_ball_image_info.width, 16)
        self.assertEqual(red_ball_image_info.height, 16)

    def test_image_info_missing(self):
        missing_image_info = ImageLibrary.image_info("@icons:does_not_exist")

        self.assertIsNone(missing_image_info)

    def test_image_info_volume_missing(self):
        missing_image_info = ImageLibrary.image_info("@missing:does_not_exist")

        self.assertIsNone(missing_image_info)

    def test_image_resource(self):
        red_ball_image = ImageLibrary.image_resource("@icons:red_ball")

        self.assertIsInstance(red_ball_image, ImageResource)
        self.assertIsInstance(red_ball_image._ref, ZipFileReference)
        self.assertTrue(ICONS_FILE.samefile(red_ball_image._ref.path))

    def test_image_resource_missing(self):
        missing_image = ImageLibrary.image_info("@icons:does_not_exist")

        self.assertIsNone(missing_image)

    def test_image_resource_missing_volume(self):
        missing_image = ImageLibrary.image_info("@missing:does_not_exist")

        self.assertIsNone(missing_image)

    def test_find_volume(self):
        volume = ImageLibrary.find_volume("@icons:red_ball")

        self.assertIsInstance(volume, ImageVolume)
        self.assertEqual(volume.name, "icons")
        self.assertTrue(ICONS_FILE.samefile(volume.path))



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from traits.api import Instance, Str

from pyface.action.action import Action
from pyface.action.i_action_manager import IActionManager


class IMenuManager(IActionManager):
    """ A menu manager realizes itself in a menu control.

    This could be a sub-menu or a context (popup) menu.
    """

    # 'IMenuManager' interface ---------------------------------------------#

    # The menu manager's name (if the manager is a sub-menu, this is what its
    # label will be).
    name = Str()

    # The default action for tool button when shown in a toolbar (Qt only)
    action = Instance(Action)

    # ------------------------------------------------------------------------
    # 'IMenuManager' interface.
    # ------------------------------------------------------------------------

    def create_menu(self, parent, controller=None):
        """ Creates a menu representation of the manager.

        Parameters
        ----------
        parent : toolkit control
            The toolkit control that owns the menu.
        controller : pyface.action.action_controller.ActionController
            An optional ActionController for all items in the menu.
        """



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


import unittest

from traits.testing.optional_dependencies import numpy as np, requires_numpy

if np is not None:
    from ..array_image import ArrayImage


@requires_numpy
class TestArrayImage(unittest.TestCase):
    def setUp(self):
        self.data = np.full((32, 64, 4), 0xee, dtype='uint8')

    def test_init(self):
        image = ArrayImage(self.data)

        self.assertIs(image.data, self.data)

    def test_init_data_required(self):
        with self.assertRaises(TypeError):
            ArrayImage()

    def test_create_image(self):
        image = ArrayImage(self.data)

        toolkit_image = image.create_image()

        self.assertIsNotNone(toolkit_image)

    def test_create_bitmap(self):
        image = ArrayImage(self.data)

        bitmap = image.create_bitmap()

        self.assertIsNotNone(bitmap)

    def test_create_icon(self):
        image = ArrayImage(self.data)

        icon = image.create_icon()

        self.assertIsNotNone(icon)



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


import unittest

from ..font import Font
from ..font_dialog import FontDialog, get_font
from ..toolkit import toolkit_object
from ..util.font_parser import simple_parser

GuiTestAssistant = toolkit_object("util.gui_test_assistant:GuiTestAssistant")
no_gui_test_assistant = GuiTestAssistant.__name__ == "Unimplemented"

ModalDialogTester = toolkit_object(
    "util.modal_dialog_tester:ModalDialogTester"
)
no_modal_dialog_tester = ModalDialogTester.__name__ == "Unimplemented"


@unittest.skipIf(no_gui_test_assistant, "No GuiTestAssistant")
class TestFontDialog(unittest.TestCase, GuiTestAssistant):

    def setUp(self):
        GuiTestAssistant.setUp(self)
        self.dialog = FontDialog(font="12 Helvetica sans-serif")

    def tearDown(self):
        if self.dialog.control is not None:
            with self.delete_widget(self.dialog.control):
                self.dialog.destroy()
        del self.dialog
        GuiTestAssistant.tearDown(self)

    def test_font(self):
        # test that fonts are translated as expected
        self.dialog.font = "10 bold condensed Helvetica sans-serif"

        self.assertFontEqual(
            self.dialog.font,
            Font(**simple_parser("10 bold condensed Helvetica sans-serif")),
        )

    def test_create(self):
        # test that creation and destruction works as expected
        with self.event_loop():
            self.dialog.create()
        with self.event_loop():
            self.dialog.destroy()

    def test_destroy(self):
        # test that destroy works even when no control
        with self.event_loop():
            self.dialog.destroy()

    def test_close(self):
        # test that close works
        with self.event_loop():
            self.dialog.create()
        with self.event_loop():
            self.dialog.close()

    def assertFontEqual(self, font1, font2):
        state1 = font1.trait_get(transient=lambda x: not x)
        state2 = font2.trait_get(transient=lambda x: not x)
        self.assertEqual(state1, state2)


@unittest.skipIf(no_gui_test_assistant, "No GuiTestAssistant")
class TestGetFont(unittest.TestCase, GuiTestAssistant):

    @unittest.skipIf(no_modal_dialog_tester, "ModalDialogTester unavailable")
    def test_close(self):
        # test that cancel works as expected
        tester = ModalDialogTester(
            lambda: get_font(None, "12 Helvetica sans-serif")
        )
        tester.open_and_wait(when_opened=lambda x: x.close(accept=False))

        self.assertEqual(tester.result, None)



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" A Layered panel. """

from traits.api import Int, provides

from pyface.qt.QtGui import QStackedWidget
from pyface.i_layered_panel import ILayeredPanel, MLayeredPanel
from .layout_widget import LayoutWidget


@provides(ILayeredPanel)
class LayeredPanel(MLayeredPanel, LayoutWidget):
    """ A Layered panel.

    A layered panel contains one or more named layers, with only one layer
    visible at any one time (think of a 'tab' control minus the tabs!).  Each
    layer is a toolkit-specific control.

    """

    # The minimum sizes of the panel.  Ignored in Qt.
    min_width = Int(0)
    min_height = Int(0)

    # ------------------------------------------------------------------------
    # 'ILayeredPanel' interface.
    # ------------------------------------------------------------------------

    def add_layer(self, name, layer):
        """ Adds a layer with the specified name.

        All layers are hidden when they are added.  Use 'show_layer' to make a
        layer visible.
        """
        self.control.addWidget(layer)
        self._layers[name] = layer
        return layer

    def show_layer(self, name):
        """ Shows the layer with the specified name. """
        layer = self._layers[name]
        layer_index = self.control.indexOf(layer)
        self.control.setCurrentIndex(layer_index)
        self.current_layer = layer
        self.current_layer_name = name
        return layer

    # ------------------------------------------------------------------------
    # "IWidget" interface.
    # ------------------------------------------------------------------------

    def _create_control(self, parent):
        """ Create the toolkit-specific control that represents the widget. """

        control = QStackedWidget(parent)
        return control



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

"""

API for the ``pyface.tasks`` submodule.

Tasks-specific Interfaces
-------------------------
- :class:`~.IDockPane`
- :class:`~pyface.tasks.i_editor.IEditor`
- :class:`~.IEditorAreaPane`
- :class:`~.ITaskPane`

Tasks, Tasks Application and related classes
--------------------------------------------

- :class:`~.AdvancedEditorAreaPane`
- :class:`~.DockPane`
- :class:`~.Editor`
- :class:`~.EditorAreaPane`
- :class:`~.SplitEditorAreaPane`
- :class:`~.Task`
- :class:`~.TasksApplication`
- :class:`~.TaskFactory`
- :class:`~.TaskPane`
- :class:`~.TaskWindow`

Tasks layout
------------
- :class:`~.TaskLayout`
- :class:`~.TaskWindowLayout`
- :class:`~.PaneItem`
- :class:`~.Tabbed`
- :class:`~.Splitter`
- :class:`~.HSplitter`
- :class:`~.VSplitter`

Traits-specific Tasks classes
-----------------------------
- :class:`~.TraitsDockPane`
- :class:`~.TraitsEditor`
- :class:`~.TraitsTaskPane`

Enaml-specific Tasks functionality
----------------------------------
- :class:`~.EnamlDockPane`
- :class:`~.EnamlEditor`
- :class:`~.EnamlTaskPane`

"""

from .i_dock_pane import IDockPane
from .i_editor import IEditor
from .i_editor_area_pane import IEditorAreaPane
from .i_task_pane import ITaskPane
from .task import Task
from .tasks_application import TasksApplication, TaskFactory
from .task_layout import (
    TaskLayout,
    PaneItem,
    Tabbed,
    Splitter,
    HSplitter,
    VSplitter,
)
from .task_window_layout import TaskWindowLayout


# ----------------------------------------------------------------------------
# Deferred imports
# ----------------------------------------------------------------------------

# These imports have the side-effect of performing toolkit selection

_toolkit_imports = {
    'AdvancedEditorAreaPane': "advanced_editor_area_pane",
    'DockPane': "dock_pane",
    'EditorAreaPane': "editor_area_pane",
    'Editor': "editor",
    'SplitEditorAreaPane': "split_editor_area_pane",
    'TaskPane': "task_pane",
}

# These are pyface.* imports that have selection as a side-effect
# TODO: refactor to delay imports where possible
_relative_imports = {
    'EnamlDockPane': "enaml_dock_pane",
    'EnamlEditor': "enaml_editor",
    'EnamlTaskPane': "enaml_task_pane",
    'TaskWindow': "task_window",
    'TraitsDockPane': "traits_dock_pane",
    'TraitsEditor': "traits_editor",
    'TraitsTaskPane': "traits_task_pane",
}


def __getattr__(name):
    """Lazily load attributes with side-effects

    In particular, lazily load toolkit backend names.  For efficiency, lazily
    loaded objects are injected into the module namespace
    """
    # sentinel object for no result
    not_found = object()
    result = not_found

    if name in _relative_imports:
        from importlib import import_module
        source = _relative_imports[name]
        module = import_module(f"pyface.tasks.{source}")
        result = getattr(module, name)

    elif name in _toolkit_imports:
        from pyface.toolkit import toolkit_object
        source = _toolkit_imports[name]
        result = toolkit_object(f"tasks.{source}:{name}")

    if result is not_found:
        raise AttributeError(f"module {__name__!r} has no attribute {name!r}")

    globals()[name] = result
    return result


# ----------------------------------------------------------------------------
# Introspection support
# ----------------------------------------------------------------------------

# the list of available names we report for introspection purposes
_extra_names = set(_toolkit_imports) | set(_relative_imports)


def __dir__():
    return sorted(set(globals()) | _extra_names)



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


import os
import sys
import warnings

from traits.etsconfig.api import ETSConfig

from pyface.ui import ShadowedModuleFinder


if any(
    (
        isinstance(finder, ShadowedModuleFinder)
        and finder.package == "pyface.ui.qt4."
    )
    for finder in sys.meta_path
):
    # Importing from pyface.ui.qt4.* is deprecated
    # Already have loader registered.
    warnings.warn(
        """The pyface.ui.qt4.* modules have moved to pyface.ui.qt.*

Backward compatibility import hooks are in place.  They will be removed in a
future release of Pyface.
""",
        DeprecationWarning,
        stacklevel=2,
    )
elif (
    os.environ.get('ETS_QT4_IMPORTS', None)  # environment says we want this
    or os.environ.get('ETS_TOOLKIT', None) == "qt4"  # environment says old qt4
    or ETSConfig.toolkit == "qt4"  # the ETSConfig toolkit says old qt4
):
    # Register our loader.  This is messing with global state that we do not own
    # so we only do it when we have other global state telling us to.

    sys.meta_path.append(ShadowedModuleFinder())

    # Importing from pyface.ui.qt4.* is deprecated
    warnings.warn(
        """The pyface.ui.qt4.* modules have moved to pyface.ui.qt.*

Backward compatibility import hooks have been automatically applied.
They will be removed in a future release of Pyface.
""",
        DeprecationWarning,
        stacklevel=2,
    )
else:
    # Don't import from this module, use a future warning as we want end-users
    # of ETS apps to see the hints about environment variables.
    warnings.warn(
        """The pyface.ui.qt4.* modules have moved to pyface.ui.qt.*.

Applications which require backwards compatibility can either:

- set the ETS_QT4_IMPORTS environment variable
- set the ETS_TOOLKIT environment variable to "qt4",
- the ETSConfig.toolkit to "qt4"
- install pyface.ui.ShadowedModuleFinder() into sys.meta_path
""",
        FutureWarning,
        stacklevel=2,
    )



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" The implementation of a dialog that allows the user to select a font.
"""

from .constant import OK
from .toolkit import toolkit_object


FontDialog = toolkit_object("font_dialog:FontDialog")


def get_font(parent, font):
    """ Convenience function that displays a font dialog.

    Parameters
    ----------
    parent : toolkit control
        The parent toolkit control for the modal dialog.
    font : Font or font description
        The initial Font object or string describing the font.

    Returns
    -------
    font : Font or None
        The selected font, or None if the user made no selection.
    """
    dialog = FontDialog(parent=parent, font=font)
    result = dialog.open()
    if result == OK:
        return dialog.font
    else:
        return None



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" The toggle field interface. """


from traits.api import Bool, HasTraits, Str

from pyface.fields.i_editable_field import IEditableField
from pyface.ui_traits import Image


class IToggleField(IEditableField):
    """ The toggle field interface.

    This is for a toggle between two states, represented by a boolean value.
    """

    #: The current value of the toggle.
    value = Bool()

    #: The text to display in the toggle.
    text = Str()

    #: The icon to display with the toggle.
    icon = Image()


class MToggleField(HasTraits):
    """ The toggle field mixin class.

    This is for a toggle between two states, represented by a boolean value.
    """

    #: The current value of the toggle.
    value = Bool()

    #: The text to display in the toggle.
    text = Str()

    #: The icon to display with the toggle.
    icon = Image()

    # ------------------------------------------------------------------------
    # Private interface
    # ------------------------------------------------------------------------

    def _initialize_control(self):
        super()._initialize_control()
        self._set_control_text(self.text)
        self._set_control_icon(self.icon)

    def _add_event_listeners(self):
        """ Set up toolkit-specific bindings for events """
        super()._add_event_listeners()
        self.observe(self._text_updated, "text", dispatch="ui")
        self.observe(self._icon_updated, "icon", dispatch="ui")

    def _remove_event_listeners(self):
        """ Remove toolkit-specific bindings for events """
        self.observe(self._text_updated, "text", dispatch="ui", remove=True)
        self.observe(self._icon_updated, "icon", dispatch="ui", remove=True)
        super()._remove_event_listeners()

    # Toolkit control interface ---------------------------------------------

    def _get_control_text(self):
        """ Toolkit specific method to set the control's text. """
        raise NotImplementedError()

    def _set_control_text(self, text):
        """ Toolkit specific method to set the control's text. """
        raise NotImplementedError()

    def _set_control_icon(self, icon):
        """ Toolkit specific method to set the control's icon. """
        raise NotImplementedError()

    # Trait change handlers -------------------------------------------------

    def _text_updated(self, event):
        if self.control is not None:
            self._set_control_text(self.text)

    def _icon_updated(self, event):
        if self.control is not None:
            self._set_control_icon(self.icon)



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" The time field interface. """

from datetime import time

from traits.api import HasTraits, Time

from pyface.fields.i_editable_field import IEditableField


class ITimeField(IEditableField):
    """ The time field interface.

    This is for a field that edits a datetime.time value.
    """

    #: The current value of the time field
    value = Time()


class MTimeField(HasTraits):
    """ Mixin class for TimeField implementations """

    #: The current value of the time field
    value = Time()

    # ------------------------------------------------------------------------
    # Private interface
    # ------------------------------------------------------------------------

    # Trait defaults --------------------------------------------------------

    def _value_default(self):
        return time.now()



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" The editable field interface. """


from traits.api import HasTraits

from .i_field import IField


class IEditableField(IField):
    """ The editable field interface.

    A editable field is a widget that displays a user-editable value.
    """


class MEditableField(HasTraits):
    """The editable field mix-in.

    Classes which use this mixin should implement _observe_control_value to
    connect a toolkit handler that calls _update_value.
    """

    # ------------------------------------------------------------------------
    # IWidget interface
    # ------------------------------------------------------------------------

    def _add_event_listeners(self):
        """ Set up toolkit-specific bindings for events """
        super()._add_event_listeners()
        self._observe_control_value()

    def _remove_event_listeners(self):
        """ Remove toolkit-specific bindings for events """
        self._observe_control_value(remove=True)
        super()._remove_event_listeners()

    # ------------------------------------------------------------------------
    # Private interface
    # ------------------------------------------------------------------------

    def _update_value(self, value):
        """ Handle a change to the value from user interaction

        This is a method suitable for calling from a toolkit event handler.
        """
        self.value = self._get_control_value()

    # Toolkit control interface ---------------------------------------------

    def _observe_control_value(self, remove=False):
        """ Toolkit specific method to change the control value observer. """
        raise NotImplementedError()



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from traits.api import HasTraits, Int, Tuple

from pyface.i_layout_item import ILayoutItem, Size, SizePolicy
from pyface.i_widget import IWidget


class ILayoutWidget(IWidget, ILayoutItem):
    """ Interface for widgets that can participate in layout.

    Most widgets implement ILayoutWidget, but widgets like top-level windows,
    menus, toolbars, etc. do not.
    """
    pass


class MLayoutWidget(HasTraits):
    """ A mixin for Widgets that can participate in layouts.

    Most widgets implement ILayoutWidget, but widgets like top-level windows,
    menus, toolbars, etc. do not.
    """

    #: The minimum size that the widget can take.
    minimum_size = Size

    #: The maximum size that the widget can take.
    maximum_size = Size

    #: Weight factor used to distribute extra space between widgets.
    stretch = Tuple(Int, Int)

    #: How the widget should behave when more space is available.
    size_policy = Tuple(SizePolicy, SizePolicy)

    def _initialize_control(self):
        """ Initialize the toolkit control. """
        super()._initialize_control()
        self._set_control_minimum_size(self.minimum_size)
        self._set_control_maximum_size(self.maximum_size)
        self._set_control_stretch(self.stretch)
        self._set_control_size_policy(self.size_policy)

    def _add_event_listeners(self):
        """ Add trait observers and toolkit binding. """
        super()._add_event_listeners()
        self.observe(
            self._minimum_size_updated,
            "minimum_size",
            dispatch="ui",
        )
        self.observe(
            self._maximum_size_updated,
            "maximum_size",
            dispatch="ui",
        )
        self.observe(
            self._stretch_updated,
            "stretch",
            dispatch="ui",
        )
        self.observe(
            self._size_policy_updated,
            "size_policy",
            dispatch="ui",
        )

    def _remove_event_listeners(self):
        """ Remove trait observers and toolkit binding. """
        self.observe(
            self._minimum_size_updated,
            "minimum_size",
            dispatch="ui",
            remove=True,
        )
        self.observe(
            self._maximum_size_updated,
            "maximum_size",
            dispatch="ui",
            remove=True,
        )
        self.observe(
            self._stretch_updated,
            "stretch",
            dispatch="ui",
            remove=True,
        )
        self.observe(
            self._size_policy_updated,
            "size_policy",
            dispatch="ui",
            remove=True,
        )
        super()._remove_event_listeners()

    def _minimum_size_updated(self, event):
        """ Trait observer for minimum size. """
        if self.control is not None:
            self._set_control_minimum_size(event.new)

    def _maximum_size_updated(self, event):
        """ Trait observer for maximum size. """
        if self.control is not None:
            self._set_control_maximum_size(event.new)

    def _stretch_updated(self, event):
        """ Trait observer for stretch. """
        if self.control is not None:
            self._set_control_stretch(event.new)

    def _size_policy_updated(self, event):
        """ Trait observer for size policy. """
        if self.control is not None:
            self._set_control_size_policy(event.new)

    def _set_control_minimum_size(self, size):
        """ Set the minimum size of the control.

        Toolkit implementations will need to override this method.
        """
        raise NotImplementedError()

    def _get_control_minimum_size(self):
        """ Get the minimum size of the control.

        Toolkit implementations will need to override this method.
        This method is only used for testing.
        """
        raise NotImplementedError()

    def _set_control_maximum_size(self, size):
        """ Set the maximum size of the control.

        Toolkit implementations will need to override this method.
        """
        raise NotImplementedError()

    def _get_control_maximum_size(self):
        """ Get the maximum size of the control.

        Toolkit implementations will need to override this method.
        This method is only used for testing.
        """
        raise NotImplementedError()

    def _set_control_stretch(self, stretch):
        """ Set the stretch factor of the control.

        Toolkit implementations will need to override this method.
        """
        raise NotImplementedError()

    def _get_control_stretch(self):
        """ Get the stretch factor of the control.

        Toolkit implementations will need to override this method.
        This method is only used for testing.
        """
        raise NotImplementedError()

    def _set_control_size_policy(self, size_policy):
        """ Set the size policy of the control.

        Toolkit implementations will need to override this method.
        """
        raise NotImplementedError()

    def _get_control_size_policy(self):
        """ Get the size policy of the control.

        Toolkit implementations will need to override this method.
        This method is only used for testing.
        """
        raise NotImplementedError()



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

"""Interface for all action managers."""


from traits.api import Bool, Constant, Event, Instance, Interface, List, Str

from pyface.action.action_controller import ActionController
from pyface.action.group import Group


class IActionManager(Interface):
    """ Abstract base class for all action managers.

    An action manager contains a list of groups, with each group containing a
    list of items.

    There are currently three concrete sub-classes:

    1) 'MenuBarManager'
    2) 'MenuManager'
    3) 'ToolBarManager'

    """

    # 'ActionManager' interface --------------------------------------------

    #: The Id of the default group.
    DEFAULT_GROUP = Constant("additions")

    #: The action controller (if any) used to control how actions are performed.
    controller = Instance(ActionController)

    #: Is the action manager enabled?
    enabled = Bool(True)

    #: All of the contribution groups in the manager.
    groups = List(Instance(Group))

    #: The manager's unique identifier (if it has one).
    id = Str()

    #: Is the action manager visible?
    visible = Bool(True)

    # Events ----

    #: fixme: We probably need more granular events than this!
    changed = Event()

    # ------------------------------------------------------------------------
    # 'object' interface.
    # ------------------------------------------------------------------------

    def __init__(self, *items, **traits):
        """ Creates a new action manager.

        Parameters
        ----------
        items : collection of strings, Group, or ActionManagerItem s
            Positional arguments are interpreted as Items or Groups
            managed by the action manager.
        traits : additional traits
            Traits to be passed to the usual Traits ``__init__``.

        Notes
        -----

        If a Group is passed as a positional agrument then it is added to the
        manager and any subsequent Items arguments are appended to the Group
        until another Group is encountered.

        If a string is passed, a Group is created with id set to the string.
        """

    # ------------------------------------------------------------------------
    # 'IActionManager' interface.
    # ------------------------------------------------------------------------

    def append(self, item):
        """ Append an item to the manager.

        Parameters
        ----------
        item : string, Group instance or ActionManagerItem
            The item to append.

        Notes
        -----

        If the item is a group, the Group is appended to the manager's list
        of groups.  It the item is a string, then a group is created with
        the string as the ``id`` and the new group is appended to the list
        of groups.  If the item is an ActionManagerItem then the item is
        appended to the manager's default group.
        """

    def destroy(self):
        """ Called when the manager is no longer required.

        By default this method simply calls 'destroy' on all of the manager's
        groups.
        """

    def insert(self, index, item):
        """ Insert an item into the manager at the specified index.

        Parameters
        ----------
        index : int
            The position at which to insert the object
        item : string, Group instance or ActionManagerItem
            The item to insert.

        Notes
        -----

        If the item is a group, the Group is inserted into the manager's list
        of groups.  It the item is a string, then a group is created with
        the string as the ``id`` and the new group is inserted into the list
        of groups.  If the item is an ActionManagerItem then the item is
        inserted into the manager's defualt group.
        """

    def find_group(self, id):
        """ Find a group with a specified Id.

        Parameters
        ----------
        id : str
            The id of the group to find.

        Returns
        -------
        group : Group
            The group which matches the id, or None if no such group exists.
        """

    def find_item(self, path):
        """ Find an item using a path.

        Parameters
        ----------
        path : str
            A '/' separated list of contribution Ids.

        Returns
        -------
        item : ActionManagerItem or None
            Returns the matching ActionManagerItem, or None if any component
            of the path is not found.
        """

    def walk(self, fn):
        """ Walk the manager applying a function at every item.

        The components are walked in pre-order.

        Parameters
        ----------
        fn : Callable
            A callable to apply to the tree of groups and items, starting with
            the manager.
        """

    def walk_group(self, group, fn):
        """ Walk a group applying a function at every item.

        The components are walked in pre-order.

        Parameters
        ----------
        group : Group
            The group to walk.
        fn : Callable
            A callable to apply to the tree of groups and items.
        """

    def walk_item(self, item, fn):
        """ Walk an item (may be a sub-menu manager remember!).

        The components are walked in pre-order.

        Parameters
        ----------
        item : item
            The item to walk.  This may be a submenu or similar in addition to
            simple Action items.
        fn : Callable
            A callable to apply to the tree of items and subgroups.
        """



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


from datetime import time
import unittest

from ..time_field import TimeField
from .field_mixin import FieldMixin


class TestTimeField(FieldMixin, unittest.TestCase):

    def _create_widget_simple(self, **traits):
        traits.setdefault("value", time(12, 0, 0))
        traits.setdefault("tooltip", "Dummy")
        return TimeField(**traits)

    # Tests ------------------------------------------------------------------

    def test_time_field(self):
        self._create_widget_control()

        self.widget.value = time(13, 1, 1)
        self.gui.process_events()

        self.assertEqual(self.widget._get_control_value(), time(13, 1, 1))

    def test_time_field_set(self):
        self._create_widget_control()

        with self.assertTraitChanges(self.widget, "value", count=1):
            self.widget._set_control_value(time(13, 1, 1))
            self.gui.process_events()

        self.assertEqual(self.widget.value, time(13, 1, 1))



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!


import unittest


from pyface.image_resource import ImageResource
from ..label_field import LabelField
from .field_mixin import FieldMixin


class TestLabelField(FieldMixin, unittest.TestCase):

    def _create_widget_simple(self, **traits):
        traits.setdefault("value", "Label")
        traits.setdefault("tooltip", "Dummy")
        return LabelField(**traits)

    # Tests ------------------------------------------------------------------

    def test_label_field(self):
        self._create_widget_control()

        self.widget.value = "Test"
        self.gui.process_events()

        self.assertEqual(self.widget._get_control_value(), "Test")

    def test_label_field_image(self):
        self._create_widget_control()
        image = ImageResource("question")

        # XXX can't validate icon values currently, so just a smoke test
        self.widget.image = image
        self.gui.process_events()



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" The Wx-specific implementation of the toggle field class """

import wx

from traits.api import provides

from pyface.fields.i_toggle_field import IToggleField, MToggleField
from .editable_field import EditableField


@provides(IToggleField)
class ToggleField(MToggleField, EditableField):
    """ The Wx-specific implementation of the toggle field class """

    # ------------------------------------------------------------------------
    # Private interface
    # ------------------------------------------------------------------------

    # Toolkit control interface ---------------------------------------------

    def _get_control_value(self):
        """ Toolkit specific method to get the control's value. """
        return self.control.GetValue()

    def _get_control_text(self):
        """ Toolkit specific method to get the control's text. """
        return self.control.GetLabel()

    def _set_control_value(self, value):
        """ Toolkit specific method to set the control's value. """
        self.control.SetValue(value)

    def _set_control_text(self, text):
        """ Toolkit specific method to set the control's text. """
        self.control.SetLabel(text)

    def _set_control_icon(self, icon):
        """ Toolkit specific method to set the control's icon. """
        # don't support icons on Wx for now
        pass


class CheckBoxField(ToggleField):
    """ The Wx-specific implementation of the checkbox class """

    def _create_control(self, parent):
        """ Create the toolkit-specific control that represents the widget. """
        control = wx.CheckBox(parent)
        return control

    def _set_control_value(self, value):
        """ Toolkit specific method to set the control's value. """
        super()._set_control_value(value)
        event = wx.CommandEvent(wx.EVT_CHECKBOX.typeId, self.control.GetId())
        event.SetInt(value)
        wx.PostEvent(self.control.GetEventHandler(), event)

    def _observe_control_value(self, remove=False):
        """ Toolkit specific method to change the control value observer. """
        if remove:
            self.control.Unbind(wx.EVT_CHECKBOX, handler=self._update_value)
        else:
            self.control.Bind(wx.EVT_CHECKBOX, self._update_value)


class RadioButtonField(ToggleField):
    """ The Wx-specific implementation of the radio button class

    This is intended to be used in groups, and shouldn't be used by itself.
    """

    def _create_control(self, parent):
        """ Create the toolkit-specific control that represents the widget. """
        control = wx.RadioButton(parent)
        return control

    def _set_control_value(self, value):
        """ Toolkit specific method to set the control's value. """
        super()._set_control_value(value)
        event = wx.CommandEvent(wx.EVT_RADIOBUTTON.typeId, self.control.GetId())
        event.SetInt(value)
        wx.PostEvent(self.control.GetEventHandler(), event)

    def _observe_control_value(self, remove=False):
        """ Toolkit specific method to change the control value observer. """
        if remove:
            self.control.Unbind(wx.EVT_RADIOBUTTON, handler=self._update_value)
        else:
            self.control.Bind(wx.EVT_RADIOBUTTON, self._update_value)


class ToggleButtonField(ToggleField):
    """ The Wx-specific implementation of the toggle button class """

    def _create_control(self, parent):
        """ Create the toolkit-specific control that represents the widget. """
        control = wx.ToggleButton(parent)
        return control

    def _set_control_value(self, value):
        """ Toolkit specific method to set the control's value. """
        super()._set_control_value(value)
        event = wx.CommandEvent(wx.EVT_TOGGLEBUTTON.typeId, self.control.GetId())
        event.SetInt(value)
        wx.PostEvent(self.control.GetEventHandler(), event)

    def _observe_control_value(self, remove=False):
        """ Toolkit specific method to change the control value observer. """
        if remove:
            self.control.Unbind(
                wx.EVT_TOGGLEBUTTON,
                handler=self._update_value,
            )
        else:
            self.control.Bind(wx.EVT_TOGGLEBUTTON, self._update_value)



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

# Note: The MultiToolbarWindow is currently wx-specific

# Import the toolkit specific version.


from .toolkit import toolkit_object

LayeredPanel = toolkit_object("layered_panel:LayeredPanel")



# (C) Copyright 2005-2023 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from unittest.mock import patch

from traits.testing.api import UnittestTools

from pyface.action.api import Action, MenuManager
from pyface.gui import GUI
from pyface.window import Window


class WidgetMixin(UnittestTools):
    """ Mixin which provides standard methods for all widgets. """

    def setUp(self):
        self.gui = GUI()

        self.parent = self._create_parent()
        self.parent.create()
        self.addCleanup(self._destroy_parent)
        self.gui.process_events()

        self.widget = self._create_widget()

        self.parent.open()
        self.gui.process_events()

    def _create_parent(self):
        return Window()

    def _create_widget(self):
        return self._create_widget_simple(parent=self.parent.control)

    def _create_widget_simple(self, **traits):
        raise NotImplementedError()

    def _create_widget_control(self):
        self.widget.create()
        self.addCleanup(self._destroy_widget)
        self.widget.show(True)
        self.gui.process_events()

    def _destroy_parent(self):
        self.parent.destroy()
        self.gui.process_events()
        self.parent = None

    def _destroy_widget(self):
        self.widget.destroy()
        self.gui.process_events()
        self.widget = None

    def test_lazy_parent_create(self):
        self.widget = self._create_widget_simple()
        self.widget.create(parent=self.parent.control)
        try:
            self.assertIsNotNone(self.widget.control)
            self.widget.show(True)
            self.gui.process_events()
        finally:
            self.widget.destroy()

    def test_widget_tooltip(self):
        self._create_widget_control()
        self.widget.tooltip = "New tooltip."
        self.gui.process_events()

        self.assertEqual(self.widget._get_control_tooltip(), "New tooltip.")

    def test_widget_tooltip_cleanup(self):
        widget = self._create_widget()
        with patch.object(widget, '_tooltip_updated', return_value=None) as updated:
            widget.create()
            try:
                widget.show(True)
                self.gui.process_events()
            finally:
                widget.destroy()
                self.gui.process_events()

            widget.tooltip = "New tooltip."

            updated.assert_not_called()

        widget = None

    def test_widget_menu(self):
        self._create_widget_control()
        self.widget.context_menu = MenuManager(Action(name="Test"), name="Test")

        self.gui.process_events()

    def test_widget_context_menu_cleanup(self):
        widget = self._create_widget()
        with patch.object(widget, '_context_menu_updated', return_value=None) as updated:
            widget.create()
            try:
                widget.show(True)
                self.gui.process_events()
            finally:
                widget.destroy()
                self.gui.process_events()

            widget.context_menu = MenuManager(Action(name="Test"), name="Test")

            updated.assert_not_called()

        widget = None
